/*
 * 	Cluster API header file.
 *	Copyright 2001 Compaq Computer Corporation
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as 
 *	published by the Free Software Foundation; either version 2 of 
 *	the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, GOOD TITLE
 *	or NON INFRINGEMENT.  See the GNU General Public License for more
 *	details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program; if not, write to the Free Software
 * 	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *	Questions/Comments/Bugfixes to ci-linux-devel@opensource.compaq.com
 *
 */
#ifndef _CLUSTER_H
#define _CLUSTER_H

#if defined(__cplusplus)
extern "C" {
#endif

#ifdef __KERNEL__
#include <linux/types.h>
#include <linux/time.h>
#else /* !__KERNEL__ */
#include <sys/types.h>
#include <sys/time.h>
#endif /* !__KERNEL__ */

typedef unsigned long clusternode_t;

/*
 * Special clusternode number values.
 */
#define CLUSTERNODE_THIS  (clusternode_t)0      /* Current node */
#define CLUSTERNODE_BEST  (clusternode_t)-1     /* Select "best" node */
#define CLUSTERNODE_INVAL (clusternode_t)-2     /* Invalid node */

/* clusternode state valid values */
#define	CLUSTERNODE_NEVERUP	0x1
#define	CLUSTERNODE_COMINGUP	0x2
#define	CLUSTERNODE_UP		0x3
#define	CLUSTERNODE_SHUTDOWN	0x4
#define	CLUSTERNODE_GOINGDOWN	0x5
#define	CLUSTERNODE_KCLEANUP	0x6
#define	CLUSTERNODE_UCLEANUP	0x7
#define	CLUSTERNODE_DOWN	0x8

/* valid action flags for setnodeinfo() */
#define	CLUSTERNODE_SETSTATE	1

/* valid values for why field in cluster_transinfo */
#define	CLUSTER_TRANSWHY_API		0x1
#define	CLUSTER_TRANSWHY_SYSTEM		0x2
#define	CLUSTER_TRANSWHY_UNEXPECTED	0x3

/* valid values for which field in cluster_detailedtransition() */
#define CLUSTER_TRANSWHICH_FIRST	0x1  /* Get first transition. */
#define CLUSTER_TRANSWHICH_NEXT		0x2  /* Get transition after transid */
#define CLUSTER_TRANSWHICH_THIS		0x3  /* Get this transid's transition */
#define CLUSTER_TRANSWHICH_LAST	0x4  /* Get latest transition */
#define CLUSTER_TRANSWHICH_PREVIOUS	0x5  /* Get transition before transid */

/* typedef unsigned long transid_t; */
typedef unsigned long transid_t;

struct cluster_transinfo {
	transid_t	transid;
	struct timeval	transtime;
	clusternode_t	transnode;
	unsigned long	fromstate;
	unsigned long	tostate;
	int		why;
};

typedef struct cluster_transinfo cluster_transinfo_t;

struct clusternode_info { 
	clusternode_t		node_num;
	unsigned long		node_state;
	unsigned long		node_previous_state;
	int			node_transwhy;
	transid_t		node_lasttransid;
	struct timeval		node_lasttranstime;
	transid_t		node_firsttransid;
	struct timeval		node_firsttranstime;
	int 			node_totalcpus;
	int			node_onlinecpus;
	int 			node_cpupower;
	int 			node_totalmemory;
};

typedef struct clusternode_info clusternode_info_t;

#ifndef __KERNEL__

#define CLUSTERNAME	"/etc/clustername"

#define NODENAME	"/etc/nodename"

#ifdef __STDC__

/* Prototypes for libcluster API's */
extern clusternode_t clusternode_num(void); 
extern char * cluster_name(void);
extern int cluster_ssiconfig(void);
extern int cluster_membership(unsigned long *, int, clusternode_t *);
extern int cluster_transition(unsigned long *, int, cluster_transinfo_t *);
extern int cluster_detailedtransition(int, unsigned long *, int, cluster_transinfo_t *);
extern int clusternode_info(clusternode_t, int, clusternode_info_t *);
extern int clusternode_setinfo(clusternode_t, int, int, clusternode_info_t *);
extern int clusternode_avail(clusternode_t);
extern int cluster_maxnodes(void);

/* System calls */
extern int rexecl (const char *path, const char *arg, ...);
extern int rexecle (const char *path, const char *arg, ...);
extern int rexeclp(const char *file, ...);
extern int rexecv(const char *file, char *const *argv, clusternode_t node);
extern int rexecvp(const char *name, char *const *argv, clusternode_t node);
extern int rfork(clusternode_t node);
extern int rfork1(clusternode_t node);
extern int rforkall(clusternode_t node);
extern int kill3(pid_t pid, int sig, int arg);
extern int migrate(clusternode_t node);

#else /*!__STDC__*/

/* Prototypes for libcluster API's */
extern clusternode_t clusternode_num(); 
extern char * cluster_name();
extern int cluster_membership();
extern int cluster_transition();
extern int cluster_detailedtransition();
extern int clusternode_info();
extern int clusternode_setinfo();
extern int clusternode_avail();
extern int cluster_maxnodes();
extern int cluster_set_config();
extern int cluster_init_preroot();
extern int cluster_init_postroot();

/* System calls */
extern int rexecl();
extern int rexecle();
extern int rexeclp();
extern int rexecv();
extern int rexecvp();
extern int rfork();
extern int rfork1();
extern int rforkall();
extern int kill3();
extern int migrate();

#endif /*__STDC__*/

#endif /* !__KERNEL__ */

#if defined(__cplusplus)
	}
#endif

#endif /* _CLUSTER_H */
